package Grenouille::Comm;

# This software may be used and distributed according to the terms
# of the GNU General Public License, incorporated herein by reference.

use strict;
use Exporter qw();
use Socket qw(:DEFAULT :crlf);
use Net::hostent qw(gethost);
use Grenouille::Vars qw(%DEBUG %VERSION);
use Grenouille::Numbers qw(jitter);
use Grenouille::Log qw(display_and_log);
use vars qw(@ISA @EXPORT_OK $SERVERNAME);
@ISA=qw(Exporter);
@EXPORT_OK=qw(contact_server pick_host_addr $SERVERNAME);

# Network
$SERVERNAME ||= 'www.grenouille.com';

my $SENDRETRIES ||= 2;

sub contact_server($) {
    my ($string) = @_;
    my ($ip, $tries, %result, $command);
    
    $command = "GET /interface.php?$string HTTP/1.0";
    if($DEBUG{'protocol'}) {
        my($display) = $command;
        $display =~ s/((?:^|&)password=)[^&]*/$1******/g;
        display_and_log('DEBUG', $display);
    }

    for($tries=0;$tries < $SENDRETRIES;$tries++) {
	eval {
	    socket(SOCK, PF_INET, SOCK_STREAM, getprotobyname('tcp')) or
		die "socket: $!";
	    $ip = inet_aton($SERVERNAME) or die "host: $!";
	    connect(SOCK, sockaddr_in(80, $ip)) or die "connect: $!";
	    
	    autoflush SOCK 1;
	    
	    print SOCK "${command}$CRLF" .
		"Host: ${SERVERNAME}$CRLF" .
		"User-Agent: $VERSION{'client'} $VERSION{'version'}$CRLF" .
		$CRLF;
	    
	    while(<SOCK>) {
		if (/([^=]+)=([^=]+)/) {
		    $result{$1} = $2;
		    chomp($result{$1});
		}
	    }
	    
	    close(SOCK);
	    display_and_log('WARNING', "Serveur: $result{'warning'}") if $result{'warning'};
	    if(not exists $result{'result'} or $result{'result'} !~ /OK/) {
                display_and_log('ERROR', "Serveur: $result{'error'}");
		die "Server responded error \"$result{'error'}\"";
            }
	};
	last unless $@;
	sleep jitter(15, 0, 5);
    }
    die if $@;
    
    return %result;
}

# randomly pick one of the IP addresses of a hostname
sub pick_host_addr($) {
    my($dns_name) = @_;
    my($h) = gethost($dns_name);
    return undef unless $h and scalar @{$h->addr_list};
    return inet_ntoa(@{$h->addr_list}[rand scalar @{$h->addr_list}]);
}

1;
