package Grenouille::Version;

# This software may be used and distributed according to the terms
# of the GNU General Public License, incorporated herein by reference.

use strict;
use Exporter qw();
use Grenouille::Vars qw(%VERSION $LASTVERSIONFILE $DAY %vars);
use Grenouille::Preferences qw(%preferences);
use Grenouille::Numbers qw(jitter);
use Grenouille::Scheduler qw(schedule_event);
use vars qw(@ISA @EXPORT_OK);
@ISA = qw(Exporter);
@EXPORT_OK = qw(display_version check_last_version);

# --------------------------- VERSION ----------------------------

sub store_version($$) {
    my($vhash, $filename) = @_;
    open(FN, ">$filename") or die "$filename: $!";
    map { print FN "$_=$vhash->{$_}\n"; } sort keys %$vhash;
    close FN;
}

sub load_version($) {
    my($filename) = @_;
    my(%vhash) = ();
    if(open(FN, "<$filename")) {
	while(<FN>) {
	    chomp;
	    $vhash{$1} = $2 if /^([^=]*)=(.*)$/;
	}
	close FN;
    }
    return %vhash;
}

sub version_to_string($) {
    my($vhash) = @_;
    my($version) = "$vhash->{'client'} $vhash->{'version'}";
    $version .= " beta $vhash->{'beta'}" if $vhash->{'beta'};
    $version .= " build $vhash->{'build'}" if $vhash->{'build'};
    $version .= " pour $vhash->{'system'}";
    return $version;
}

sub compare_versions($$) {
    my($vhash1, $vhash2) = @_;
    my($cmp);
    return undef if $vhash1->{'client'} ne $vhash2->{'client'};
    # will become wrong when plgrenouille will run on different systems ?
    return undef if $vhash1->{'system'} ne $vhash2->{'system'};
    $cmp = ($vhash1->{'version'} <=> $vhash2->{'version'});
    return $cmp if $cmp;
    return -1 if $vhash1->{'beta'} and not $vhash2->{'beta'};
    return 1 if not $vhash1->{'beta'} and $vhash2->{'beta'};
    $cmp = ($vhash1->{'beta'} <=> $vhash2->{'beta'});
    return $cmp if $cmp;
    $cmp = ($vhash1->{'build'} <=> $vhash2->{'build'});
    return $cmp;
}

sub display_version() {
    # ASCII art by Donna Shepherd
    print "\n",
    "         @..@\n",
    '        (\--/)               ', version_to_string(\%VERSION), "\n",
    "       (.>__<.)\n",
    "       ^^^  ^^^\n\n";
}

sub check_last_version($) { # EVENT
    my($schedule_time) = shift;
    return unless $vars{'id'};
    
    my(%version);
    eval {
	%version = contact_server("command=get_last_version&id=$vars{'id'}");
    };
    if (not $@) {
        my($check);
	# 0 looks like a better special case value than 255 for my taste
	$version{'beta'} = 0 if $version{'beta'} == 255;
	display_and_log('DISPLAY', "Dernire version disponible : " . version_to_string(\%version));
	$check = compare_versions(\%VERSION, \%version);
	if(not defined $check) {
	    display_and_log('WARNING', "Quelque chose de bizarre est en train d'arriver...");
	} else {
	    if($check < 0) {
		my($message) = ($version{'beta'} and not $VERSION{'beta'}) ?
		    "Une nouvelle version de test de plgrenouille est disponible,\nmettez  jour uniquement si vous voulez tester un logiciel en phase bta...\n" :
			"Une nouvelle version de plgrenouille est disponible,\nil est conseill de procder  une mise  jour...\n";
		if(-t STDOUT or not $preferences{'email_notification'}) {
		    # interactive usage, just report the good news
		    display_and_log('DISPLAY', $message);
		} else {
		    # otherwise, notify by email ?
		    my(%last_notified_version) = load_version($LASTVERSIONFILE);
		    if((not %last_notified_version or compare_versions(\%last_notified_version, \%version)) and 
		       open(MAIL, "|mail -s \"" . version_to_string(\%version) . "\" $preferences{'email'}")) {
			print MAIL $message;
			close MAIL;
			store_version(\%version, $LASTVERSIONFILE);
		    }
		}
	    }
	    display_and_log('DISPLAY', 'Super, votre version est plus rcente que celle de grenouille.com ! :)') if $check > 0;
	}
    }
    schedule_event($schedule_time + jitter(1 * $DAY, -1, 1), \&check_last_version);
}

1;
